@@ -1,45 +1,45 @@
-use std::char;
-use std::str;
-use std::cmp;
+use core::char;
+use core::cmp;
+use core::str;
 
 use {
-    TextPos,
     StreamError,
     StrSpan,
+    TextPos,
     XmlByteExt,
     XmlCharExt,
 };
 
-type Result<T> = ::std::result::Result<T, StreamError>;
+type Result<T> = ::core::result::Result<T, StreamError>;
 
 
 /// Representation of the [Reference](https://www.w3.org/TR/xml/#NT-Reference) value.
-#[derive(PartialEq, Clone, Copy, Debug)]
+#[derive(Clone, Copy, PartialEq, Debug)]
 pub enum Reference<'a> {
     /// An entity reference.
     ///
     /// <https://www.w3.org/TR/xml/#NT-EntityRef>
-    EntityRef(&'a str),
+    Entity(&'a str),
+
     /// A character reference.
     ///
     /// <https://www.w3.org/TR/xml/#NT-CharRef>
-    CharRef(char),
+    Char(char),
 }
 
 
-/// A streaming text parsing interface.
-#[derive(PartialEq, Clone, Copy, Debug)]
+/// A streaming XML parsing interface.
+#[derive(Clone, Copy, PartialEq)]
 pub struct Stream<'a> {
-    bytes: &'a [u8],
     pos: usize,
     end: usize,
     span: StrSpan<'a>,
 }
 
 impl<'a> From<&'a str> for Stream<'a> {
+    #[inline]
     fn from(text: &'a str) -> Self {
         Stream {
-            bytes: text.as_bytes(),
             pos: 0,
             end: text.len(),
             span: text.into(),
@@ -48,11 +48,11 @@ impl<'a> From<&'a str> for Stream<'a> {
 }
 
 impl<'a> From<StrSpan<'a>> for Stream<'a> {
+    #[inline]
     fn from(span: StrSpan<'a>) -> Self {
         Stream {
-            bytes: span.to_str().as_bytes(),
             pos: 0,
-            end: span.len(),
+            end: span.as_str().len(),
             span,
         }
     }
@@ -60,11 +60,13 @@ impl<'a> From<StrSpan<'a>> for Stream<'a> {
 
 impl<'a> Stream<'a> {
     /// Returns an underling string span.
+    #[inline]
     pub fn span(&self) -> StrSpan<'a> {
         self.span
     }
 
     /// Returns current position.
+    #[inline]
     pub fn pos(&self) -> usize {
         self.pos
     }
@@ -72,6 +74,7 @@ impl<'a> Stream<'a> {
     /// Sets current position equal to the end.
     ///
     /// Used to indicate end of parsing on error.
+    #[inline]
     pub fn jump_to_end(&mut self) {
         self.pos = self.end;
     }
@@ -97,6 +100,7 @@ impl<'a> Stream<'a> {
     /// # Errors
     ///
     /// - `UnexpectedEndOfStream`
+    #[inline]
     pub fn curr_byte(&self) -> Result<u8> {
         if self.at_end() {
             return Err(StreamError::UnexpectedEndOfStream);
@@ -112,29 +116,7 @@ impl<'a> Stream<'a> {
     /// - if the current position is after the end of the data
     #[inline]
     pub fn curr_byte_unchecked(&self) -> u8 {
-        self.bytes[self.pos]
-    }
-
-    /// Checks that current byte is equal to provided.
-    ///
-    /// Returns `false` if no bytes left.
-    #[inline]
-    pub fn is_curr_byte_eq(&self, c: u8) -> bool {
-        if !self.at_end() {
-            self.curr_byte_unchecked() == c
-        } else {
-            false
-        }
-    }
-
-    /// Returns a byte from a current stream position if there is one.
-    #[inline]
-    pub fn get_curr_byte(&self) -> Option<u8> {
-        if !self.at_end() {
-            Some(self.curr_byte_unchecked())
-        } else {
-            None
-        }
+        self.span.as_bytes()[self.pos]
     }
 
     /// Returns a next byte from a current stream position.
@@ -142,30 +124,13 @@ impl<'a> Stream<'a> {
     /// # Errors
     ///
     /// - `UnexpectedEndOfStream`
+    #[inline]
     pub fn next_byte(&self) -> Result<u8> {
         if self.pos + 1 >= self.end {
             return Err(StreamError::UnexpectedEndOfStream);
         }
 
-        Ok(self.bytes[self.pos + 1])
-    }
-
-    /// Returns a char from a current stream position.
-    ///
-    /// # Errors
-    ///
-    /// - `UnexpectedEndOfStream`
-    pub fn curr_char(&self) -> Result<char> {
-        if self.at_end() {
-            return Err(StreamError::UnexpectedEndOfStream);
-        }
-
-        Ok(self.curr_char_unchecked())
-    }
-
-    #[inline]
-    fn curr_char_unchecked(&self) -> char {
-        self.span.to_str()[self.pos..].chars().next().unwrap()
+        Ok(self.span.as_bytes()[self.pos + 1])
     }
 
     /// Advances by `n` bytes.
@@ -185,58 +150,6 @@ impl<'a> Stream<'a> {
         self.pos += n;
     }
 
-    /// Skips whitespaces.
-    ///
-    /// Accepted values: `' ' \n \r \t &#x20; &#x9; &#xD; &#xA;`.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use xmlparser::Stream;
-    ///
-    /// let mut s = Stream::from(" \t\n\r &#x20; ");
-    /// s.skip_spaces();
-    /// assert_eq!(s.at_end(), true);
-    /// ```
-    pub fn skip_spaces(&mut self) {
-        while !self.at_end() {
-            let c = self.curr_byte_unchecked();
-
-            if c.is_xml_space() {
-                self.advance(1);
-            } else if c == b'&' {
-                // Check for (#x20 | #x9 | #xD | #xA).
-                let start = self.pos();
-                let mut is_space = false;
-                if let Ok(Reference::CharRef(ch)) = self.consume_reference() {
-                    if (ch as u32) < 255 && (ch as u8).is_xml_space() {
-                        is_space = true;
-                    }
-                }
-
-                if !is_space {
-                    self.pos = start;
-                    break;
-                }
-            } else {
-                break;
-            }
-        }
-    }
-
-    /// Skips ASCII whitespaces.
-    ///
-    /// Accepted values: `' ' \n \r \t`.
-    pub fn skip_ascii_spaces(&mut self) {
-        while !self.at_end() {
-            if self.curr_byte_unchecked().is_xml_space() {
-                self.advance(1);
-            } else {
-                break;
-            }
-        }
-    }
-
     /// Checks that the stream starts with a selected text.
     ///
     /// We are using `&[u8]` instead of `&str` for performance reasons.
@@ -253,57 +166,10 @@ impl<'a> Stream<'a> {
     /// ```
     #[inline]
     pub fn starts_with(&self, text: &[u8]) -> bool {
-        self.bytes[self.pos..self.end].starts_with(text)
+        self.span.as_bytes()[self.pos..self.end].starts_with(text)
     }
 
-    /// Checks if the stream is starts with a space.
-    ///
-    /// Uses [`skip_spaces()`](#method.curr_byte) internally.
-    pub fn starts_with_space(&self) -> bool {
-        if self.at_end() {
-            return false;
-        }
-
-        let mut is_space = false;
-
-        let c = self.curr_byte_unchecked();
-
-        if c.is_xml_space() {
-            is_space = true;
-        } else if c == b'&' {
-            // Check for (#x20 | #x9 | #xD | #xA).
-            let mut s = *self;
-            if let Some(Reference::CharRef(v)) = s.try_consume_reference() {
-                if (v as u32) < 255 && (v as u8).is_xml_space() {
-                    is_space = true;
-                }
-            }
-        }
-
-        is_space
-    }
-
-    /// Consumes whitespaces.
-    ///
-    /// Like [`skip_spaces()`], but checks that first char is actually a space.
-    ///
-    /// [`skip_spaces()`]: #method.skip_spaces
-    ///
-    /// # Errors
-    ///
-    /// - `InvalidChar`
-    pub fn consume_spaces(&mut self) -> Result<()> {
-        if !self.at_end() && !self.starts_with_space() {
-            let c = self.curr_byte_unchecked() as char;
-            let pos = self.gen_text_pos();
-            return Err(StreamError::InvalidSpace(c, pos));
-        }
-
-        self.skip_spaces();
-        Ok(())
-    }
-
-    /// Consumes current byte if it's equal to the provided byte.
+    /// Consumes the current byte if it's equal to the provided byte.
     ///
     /// # Errors
     ///
@@ -316,72 +182,282 @@ impl<'a> Stream<'a> {
     /// use xmlparser::Stream;
     ///
     /// let mut s = Stream::from("Some text.");
-    /// s.consume_byte(b'S').unwrap();
-    /// s.consume_byte(b'o').unwrap();
-    /// s.consume_byte(b'm').unwrap();
-    /// // s.consume_byte(b'q').unwrap(); // will produce an error
+    /// assert!(s.consume_byte(b'S').is_ok());
+    /// assert!(s.consume_byte(b'o').is_ok());
+    /// assert!(s.consume_byte(b'm').is_ok());
+    /// assert!(s.consume_byte(b'q').is_err());
     /// ```
     pub fn consume_byte(&mut self, c: u8) -> Result<()> {
-        if self.curr_byte()? != c {
-            return Err(
-                StreamError::InvalidChar(
-                    vec![self.curr_byte_unchecked(), c],
-                    self.gen_text_pos(),
-                )
-            );
+        let curr = self.curr_byte()?;
+        if curr != c {
+            return Err(StreamError::InvalidChar(curr, c, self.gen_text_pos()));
         }
 
         self.advance(1);
         Ok(())
     }
 
-    /// Consumes current byte if it's equal to one of the provided bytes.
+    /// Tries to consume the current byte if it's equal to the provided byte.
     ///
-    /// Returns a coincidental byte.
-    ///
-    /// # Errors
-    ///
-    /// - `InvalidChar`
-    /// - `UnexpectedEndOfStream`
-    pub fn consume_either(&mut self, list: &[u8]) -> Result<u8> {
-        assert!(!list.is_empty());
-
-        let c = self.curr_byte()?;
-        if !list.contains(&c) {
-            let mut v = list.to_vec();
-            v.insert(0, c);
-            return Err(StreamError::InvalidChar(v, self.gen_text_pos()));
+    /// Unlike `consume_byte()` will not return any errors.
+    pub fn try_consume_byte(&mut self, c: u8) -> bool {
+        match self.curr_byte() {
+            Ok(b) if b == c => {
+                self.advance(1);
+                true
+            }
+            _ => false,
         }
-
-        self.advance(1);
-        Ok(c)
     }
 
-    /// Consumes selected string.
+    /// Skips selected string.
     ///
     /// # Errors
     ///
-    /// - `InvalidChar`
-    pub fn skip_string(&mut self, text: &[u8]) -> Result<()> {
+    /// - `InvalidString`
+    pub fn skip_string(&mut self, text: &'static [u8]) -> Result<()> {
         if !self.starts_with(text) {
-            let len = cmp::min(text.len(), self.end - self.pos);
-            // Collect chars and do not slice a string,
-            // because the `len` can be on the char boundary.
-            // Which lead to a panic.
-            let actual = self.span.to_str()[self.pos..].chars().take(len).collect();
-
-            // Assume that all input `text` are valid UTF-8 strings, so unwrap is safe.
-            let expected = str::from_utf8(text).unwrap().to_owned();
-
             let pos = self.gen_text_pos();
 
-            return Err(StreamError::InvalidString(vec![actual, expected], pos));
+            // Assume that all input `text` are valid UTF-8 strings, so unwrap is safe.
+            let expected = str::from_utf8(text).unwrap();
+
+            return Err(StreamError::InvalidString(expected, pos));
         }
 
         self.advance(text.len());
         Ok(())
     }
 
+    /// Consumes bytes by the predicate and returns them.
+    ///
+    /// The result can be empty.
+    #[inline]
+    pub fn consume_bytes<F>(&mut self, f: F) -> StrSpan<'a>
+        where F: Fn(&Stream, u8) -> bool
+    {
+        let start = self.pos;
+        self.skip_bytes(f);
+        self.slice_back(start)
+    }
+
+    /// Skips bytes by the predicate.
+    pub fn skip_bytes<F>(&mut self, f: F)
+        where F: Fn(&Stream, u8) -> bool
+    {
+        while !self.at_end() && f(self, self.curr_byte_unchecked()) {
+            self.advance(1);
+        }
+    }
+
+    /// Consumes chars by the predicate and returns them.
+    ///
+    /// The result can be empty.
+    pub fn consume_chars<F>(&mut self, f: F) -> StrSpan<'a>
+        where F: Fn(&Stream, char) -> bool
+    {
+        let start = self.pos;
+        self.skip_chars(f);
+        self.slice_back(start)
+    }
+
+    /// Skips chars by the predicate.
+    pub fn skip_chars<F>(&mut self, f: F)
+        where F: Fn(&Stream, char) -> bool
+    {
+        for c in self.chars() {
+            if f(self, c) {
+                self.advance(c.len_utf8());
+            } else {
+                break;
+            }
+        }
+    }
+
+    #[inline]
+    pub(crate) fn chars(&self) -> str::Chars<'a> {
+        self.span.as_str()[self.pos..self.end].chars()
+    }
+
+    /// Slices data from `pos` to the current position.
+    #[inline]
+    pub fn slice_back(&self, pos: usize) -> StrSpan<'a> {
+        self.span.slice_region(pos, self.pos)
+    }
+
+    /// Slices data from the current position to the end.
+    #[inline]
+    pub fn slice_tail(&self) -> StrSpan<'a> {
+        self.span.slice_region(self.pos, self.end)
+    }
+
+    /// Calculates a current absolute position.
+    ///
+    /// This operation is very expensive. Use only for errors.
+    #[inline(never)]
+    pub fn gen_text_pos(&self) -> TextPos {
+        let text = self.span.full_str();
+        let end = self.pos + self.span.start();
+
+        let row = Self::calc_curr_row(text, end);
+        let col = Self::calc_curr_col(text, end);
+        TextPos::new(row, col)
+    }
+
+    /// Calculates an absolute position at `pos`.
+    ///
+    /// This operation is very expensive. Use only for errors.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// let s = xmlparser::Stream::from("text");
+    ///
+    /// assert_eq!(s.gen_text_pos_from(2), xmlparser::TextPos::new(1, 3));
+    /// assert_eq!(s.gen_text_pos_from(9999), xmlparser::TextPos::new(1, 5));
+    /// ```
+    #[inline(never)]
+    pub fn gen_text_pos_from(&self, pos: usize) -> TextPos {
+        let mut s = self.clone();
+        s.pos = cmp::min(pos, s.span.full_str().len());
+        s.gen_text_pos()
+    }
+
+    fn calc_curr_row(text: &str, end: usize) -> u32 {
+        let mut row = 1;
+        for c in &text.as_bytes()[..end] {
+            if *c == b'\n' {
+                row += 1;
+            }
+        }
+
+        row
+    }
+
+    fn calc_curr_col(text: &str, end: usize) -> u32 {
+        let mut col = 1;
+        for c in text[..end].chars().rev() {
+            if c == '\n' {
+                break;
+            } else {
+                col += 1;
+            }
+        }
+
+        col
+    }
+
+    /// Skips whitespaces.
+    ///
+    /// Accepted values: `' ' \n \r \t`.
+    #[inline]
+    pub fn skip_spaces(&mut self) {
+        while !self.at_end() && self.curr_byte_unchecked().is_xml_space() {
+            self.advance(1);
+        }
+    }
+
+    /// Checks if the stream is starts with a space.
+    #[inline]
+    pub fn starts_with_space(&self) -> bool {
+        !self.at_end() && self.curr_byte_unchecked().is_xml_space()
+    }
+
+    /// Consumes whitespaces.
+    ///
+    /// Like [`skip_spaces()`], but checks that first char is actually a space.
+    ///
+    /// [`skip_spaces()`]: #method.skip_spaces
+    ///
+    /// # Errors
+    ///
+    /// - `InvalidSpace`
+    pub fn consume_spaces(&mut self) -> Result<()> {
+        if self.at_end() {
+            return Err(StreamError::UnexpectedEndOfStream);
+        }
+
+        if !self.starts_with_space() {
+            let c = self.curr_byte_unchecked() as char;
+            let pos = self.gen_text_pos();
+            return Err(StreamError::InvalidSpace(c, pos));
+        }
+
+        self.skip_spaces();
+        Ok(())
+    }
+
+    /// Consumes an XML character reference if there is one.
+    ///
+    /// On error will reset the position to the original.
+    pub fn try_consume_reference(&mut self) -> Option<Reference<'a>> {
+        let start = self.pos();
+
+        // Consume reference on a substream.
+        let mut s = self.clone();
+        match s.consume_reference() {
+            Ok(r) => {
+                // If the current data is a reference than advance the current stream
+                // by number of bytes read by substream.
+                self.advance(s.pos() - start);
+                Some(r)
+            }
+            Err(_) => {
+                None
+            }
+        }
+    }
+
+    /// Consumes an XML reference.
+    ///
+    /// Consumes according to: <https://www.w3.org/TR/xml/#NT-Reference>
+    ///
+    /// # Errors
+    ///
+    /// - `InvalidReference`
+    pub fn consume_reference(&mut self) -> Result<Reference<'a>> {
+        self._consume_reference().map_err(|_| StreamError::InvalidReference)
+    }
+
+    fn _consume_reference(&mut self) -> Result<Reference<'a>> {
+        if !self.try_consume_byte(b'&') {
+            return Err(StreamError::InvalidReference);
+        }
+
+        let reference = if self.try_consume_byte(b'#') {
+            let (value, radix) = if self.try_consume_byte(b'x') {
+                let value = self.consume_bytes(|_, c| c.is_xml_hex_digit()).as_str();
+                (value, 16)
+            } else {
+                let value = self.consume_bytes(|_, c| c.is_xml_digit()).as_str();
+                (value, 10)
+            };
+
+            let n = u32::from_str_radix(value, radix).map_err(|_| StreamError::InvalidReference)?;
+
+            let c = char::from_u32(n).unwrap_or('\u{FFFD}');
+            if !c.is_xml_char() {
+                return Err(StreamError::InvalidReference);
+            }
+
+            Reference::Char(c)
+        } else {
+            let name = self.consume_name()?;
+            match name.as_str() {
+                "quot" => Reference::Char('"'),
+                "amp"  => Reference::Char('&'),
+                "apos" => Reference::Char('\''),
+                "lt"   => Reference::Char('<'),
+                "gt"   => Reference::Char('>'),
+                _ => Reference::Entity(name.as_str()),
+            }
+        };
+
+        self.consume_byte(b';')?;
+
+        Ok(reference)
+    }
+
     /// Consumes an XML name and returns it.
     ///
     /// Consumes according to: <https://www.w3.org/TR/xml/#NT-Name>
@@ -409,9 +485,8 @@ impl<'a> Stream<'a> {
     /// # Errors
     ///
     /// - `InvalidName` - if name is empty or starts with an invalid char
-    /// - `UnexpectedEndOfStream`
     pub fn skip_name(&mut self) -> Result<()> {
-        let mut iter = self.span.to_str()[self.pos..self.end].chars();
+        let mut iter = self.chars();
         if let Some(c) = iter.next() {
             if c.is_xml_name_start() {
                 self.advance(c.len_utf8());
@@ -438,25 +513,41 @@ impl<'a> Stream<'a> {
     /// # Errors
     ///
     /// - `InvalidName` - if name is empty or starts with an invalid char
-    /// - `UnexpectedEndOfStream`
     pub fn consume_qname(&mut self) -> Result<(StrSpan<'a>, StrSpan<'a>)> {
         let start = self.pos();
 
         let mut splitter = None;
-        let iter = self.span.to_str()[self.pos..self.end].chars();
-        for c in iter {
-            if c == ':' {
-                splitter = Some(self.pos());
-                self.advance(1);
-            } else if c.is_xml_name() {
-                self.advance(c.len_utf8());
+
+        while !self.at_end() {
+            // Check for ASCII first for performance reasons.
+            let b = self.curr_byte_unchecked();
+            if b < 128 {
+                if b == b':' {
+                    if splitter.is_none() {
+                        splitter = Some(self.pos());
+                        self.advance(1);
+                    } else {
+                        // Multiple `:` is an error.
+                        return Err(StreamError::InvalidName);
+                    }
+                } else if b.is_xml_name() {
+                    self.advance(1);
+                } else {
+                    break;
+                }
             } else {
-                break;
+                // Fallback to Unicode code point.
+                match self.chars().nth(0) {
+                    Some(c) if c.is_xml_name() => {
+                        self.advance(c.len_utf8());
+                    }
+                    _ => break,
+                }
             }
         }
 
         let (prefix, local) = if let Some(splitter) = splitter {
-            let prefix = self.span.slice_region(start, splitter);
+            let prefix = self.span().slice_region(start, splitter);
             let local = self.slice_back(splitter + 1);
             (prefix, local)
         } else {
@@ -464,7 +555,20 @@ impl<'a> Stream<'a> {
             ("".into(), local)
         };
 
-        if local.is_empty() {
+        // Prefix must start with a `NameStartChar`.
+        if let Some(c) = prefix.as_str().chars().nth(0) {
+            if !c.is_xml_name_start() {
+                return Err(StreamError::InvalidName);
+            }
+        }
+
+        // Local name must start with a `NameStartChar`.
+        if let Some(c) = local.as_str().chars().nth(0) {
+            if !c.is_xml_name_start() {
+                return Err(StreamError::InvalidName);
+            }
+        } else {
+            // If empty - error.
             return Err(StreamError::InvalidName);
         }
 
@@ -478,10 +582,11 @@ impl<'a> Stream<'a> {
     /// # Errors
     ///
     /// - `InvalidChar`
+    /// - `UnexpectedEndOfStream`
     pub fn consume_eq(&mut self) -> Result<()> {
-        self.skip_ascii_spaces();
+        self.skip_spaces();
         self.consume_byte(b'=')?;
-        self.skip_ascii_spaces();
+        self.skip_spaces();
 
         Ok(())
     }
@@ -503,219 +608,4 @@ impl<'a> Stream<'a> {
             Err(StreamError::InvalidQuote(c as char, self.gen_text_pos()))
         }
     }
-
-    /// Consumes bytes by the predicate and returns them.
-    ///
-    /// The result can be empty.
-    pub fn consume_bytes<F>(&mut self, f: F) -> StrSpan<'a>
-        where F: Fn(&Stream, u8) -> bool
-    {
-        let start = self.pos();
-        self.skip_bytes(f);
-        self.slice_back(start)
-    }
-
-    /// Consumes bytes by the predicate.
-    pub fn skip_bytes<F>(&mut self, f: F)
-        where F: Fn(&Stream, u8) -> bool
-    {
-        while !self.at_end() {
-            let c = self.curr_byte_unchecked();
-            if f(self, c) {
-                self.advance(1);
-            } else {
-                break;
-            }
-        }
-    }
-
-    /// Consumes chars by the predicate and returns them.
-    ///
-    /// The result can be empty.
-    pub fn consume_chars<F>(&mut self, f: F) -> StrSpan<'a>
-        where F: Fn(&Stream, char) -> bool
-    {
-        let start = self.pos();
-        self.skip_chars(f);
-        self.slice_back(start)
-    }
-
-    /// Consumes chars by the predicate.
-    pub fn skip_chars<F>(&mut self, f: F)
-        where F: Fn(&Stream, char) -> bool
-    {
-        let t = &self.span.to_str()[self.pos..self.end];
-        for c in t.chars() {
-            if f(self, c) {
-                self.advance(c.len_utf8());
-            } else {
-                break;
-            }
-        }
-    }
-
-    /// Consumes an XML character reference if there is one.
-    ///
-    /// On error will reset the position to the original.
-    pub fn try_consume_reference(&mut self) -> Option<Reference<'a>> {
-        let start = self.pos();
-
-        match self.consume_reference() {
-            Ok(r) => Some(r),
-            Err(_) => {
-                self.pos = start;
-                None
-            }
-        }
-    }
-
-    /// Consumes an XML reference.
-    ///
-    /// Consumes according to: <https://www.w3.org/TR/xml/#NT-Reference>
-    ///
-    /// # Errors
-    ///
-    /// - `InvalidReference`
-    /// - `UnexpectedEndOfStream`
-    pub fn consume_reference(&mut self) -> Result<Reference<'a>> {
-        self._consume_reference().map_err(|_| StreamError::InvalidReference)
-    }
-
-    fn _consume_reference(&mut self) -> Result<Reference<'a>> {
-        if self.curr_byte()? != b'&' {
-            return Err(StreamError::InvalidReference);
-        }
-
-        self.advance(1);
-        let reference = if self.curr_byte()? == b'#' {
-            self.advance(1);
-            let n = if self.curr_byte()? == b'x' {
-                self.advance(1);
-                let value = self.consume_bytes(|_, c| c.is_xml_hex_digit()).to_str();
-                u32::from_str_radix(value, 16).map_err(|_| StreamError::InvalidReference)
-            } else {
-                let value = self.consume_bytes(|_, c| c.is_xml_digit()).to_str();
-                u32::from_str_radix(value, 10).map_err(|_| StreamError::InvalidReference)
-            }?;
-
-            let c = char::from_u32(n).unwrap_or('\u{FFFD}');
-            if !c.is_xml_char() {
-                return Err(StreamError::InvalidReference);
-            }
-
-            Reference::CharRef(c)
-        } else {
-            let name = self.consume_name()?;
-            match name.to_str() {
-                "quot" => Reference::CharRef('"'),
-                "amp"  => Reference::CharRef('&'),
-                "apos" => Reference::CharRef('\''),
-                "lt"   => Reference::CharRef('<'),
-                "gt"   => Reference::CharRef('>'),
-                _ => Reference::EntityRef(name.to_str()),
-            }
-        };
-
-        self.consume_byte(b';')?;
-
-        Ok(reference)
-    }
-
-    /// Slices data from `pos` to the current position.
-    pub fn slice_back(&self, pos: usize) -> StrSpan<'a> {
-        self.span.slice_region(pos, self.pos())
-    }
-
-    /// Slices data from the current position to the end.
-    pub fn slice_tail(&self) -> StrSpan<'a> {
-        self.span.slice_region(self.pos(), self.end)
-    }
-
-    /// Calculates a current absolute position.
-    ///
-    /// This operation is very expensive. Use only for errors.
-    #[inline(never)]
-    pub fn gen_text_pos(&self) -> TextPos {
-        let row = self.calc_curr_row();
-        let col = self.calc_curr_col();
-        TextPos::new(row, col)
-    }
-
-    /// Calculates an absolute position at `pos`.
-    ///
-    /// This operation is very expensive. Use only for errors.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// let s = xmlparser::Stream::from("text");
-    ///
-    /// assert_eq!(s.gen_text_pos_from(2), xmlparser::TextPos::new(1, 3));
-    /// assert_eq!(s.gen_text_pos_from(9999), xmlparser::TextPos::new(1, 5));
-    /// ```
-    #[inline(never)]
-    pub fn gen_text_pos_from(&self, pos: usize) -> TextPos {
-        let mut s = *self;
-        let old_pos = s.pos;
-
-        s.pos = cmp::min(pos, s.span.full_str().len());
-        let e = s.gen_text_pos();
-        s.pos = old_pos;
-
-        e
-    }
-
-    fn calc_curr_row(&self) -> u32 {
-        let text = self.span.full_str();
-        let end = self.pos + self.span.start();
-        let mut row = 1;
-        for c in &text.as_bytes()[..end] {
-            if *c == b'\n' {
-                row += 1;
-            }
-        }
-
-        row
-    }
-
-    fn calc_curr_col(&self) -> u32 {
-        let text = self.span.full_str();
-        let end = self.pos + self.span.start();
-        let mut col = 1;
-        for c in text[..end].chars().rev() {
-            if c == '\n' {
-                break;
-            } else {
-                col += 1;
-            }
-        }
-
-        col
-    }
-}
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[test]
-    fn text_pos_1() {
-        let mut s = Stream::from("text");
-        s.advance(2);
-        assert_eq!(s.gen_text_pos(), TextPos::new(1, 3));
-    }
-
-    #[test]
-    fn text_pos_2() {
-        let mut s = Stream::from("text\ntext");
-        s.advance(6);
-        assert_eq!(s.gen_text_pos(), TextPos::new(2, 2));
-    }
-
-    #[test]
-    fn text_pos_3() {
-        let mut s = Stream::from("текст\nтекст");
-        s.advance(15);
-        assert_eq!(s.gen_text_pos(), TextPos::new(2, 3));
-    }
 }
